/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "passutils.h"
#include <QString>
#include <QObject>
#include <QInputDialog>
#include <util/xpc/util.h>
#include "interfaces/structures.h"

namespace LeechCraft
{
namespace Util
{
	namespace
	{
		QString GetPasswordHelper (const QString& key, QObject *emitter)
		{
			QList<QVariant> keys;
			keys << key;
			const auto& result = Util::GetPersistentData (keys, emitter);
			if (result.size () != 1)
			{
				qWarning () << Q_FUNC_INFO
						<< "incorrect result size for key"
						<< key
						<< "; result:"
						<< result;
				return QString ();
			}

			const auto& strVarList = result.at (0).toList ();
			if (strVarList.isEmpty () ||
					!strVarList.at (0).canConvert<QString> ())
			{
				qWarning () << Q_FUNC_INFO
						<< "invalid string variant list"
						<< strVarList
						<< "for key"
						<< key;
				return QString ();
			}

			return strVarList.at (0).toString ();
		}
	}

	QString GetPassword (const QString& key, const QString& diaText,
			QObject *emitter, bool useStored)
	{
		if (useStored)
		{
			const QString& result = GetPasswordHelper (key, emitter);
			if (!result.isNull ())
				return result;
		}

		QString result = QInputDialog::getText (0,
				"LeechCraft",
				diaText,
				QLineEdit::Password);
		if (!result.isNull ())
			SavePassword (result, key, emitter);
		return result;
	}

	void SavePassword (const QString& password, const QString& key,
			QObject *emitter)
	{
		QList<QVariant> keys;
		keys << key;

		QList<QVariant> passwordVar;
		passwordVar << password;
		QList<QVariant> values;
		values << QVariant (passwordVar);

		Entity e = Util::MakeEntity (keys,
				QString (),
				Internal,
				"x-leechcraft/data-persistent-save");
		e.Additional_ ["Values"] = values;
		e.Additional_ ["Overwrite"] = true;

		QMetaObject::invokeMethod (emitter, "gotEntity", Q_ARG (LeechCraft::Entity, e));
	}
}
}
